@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-#
 # Copyright 2010-2019 The pygit2 contributors
 #
 # This file is free software; you can redistribute it and/or modify
@@ -25,28 +23,21 @@
 # the Free Software Foundation, 51 Franklin Street, Fifth Floor,
 # Boston, MA 02110-1301, USA.
 
-# Import from the future
-from __future__ import absolute_import
-
 # Import from the Standard Library
+from io import BytesIO
 from string import hexdigits
-import sys, tarfile
+import tarfile
 from time import time
 
-if sys.version_info[0] < 3:
-    from cStringIO import StringIO
-else:
-    from io import BytesIO as StringIO
-
-import six
-
 # Import from pygit2
 from _pygit2 import Repository as _Repository, init_file_backend
 from _pygit2 import Oid, GIT_OID_HEXSZ, GIT_OID_MINPREFIXLEN
 from _pygit2 import GIT_CHECKOUT_SAFE, GIT_CHECKOUT_RECREATE_MISSING, GIT_DIFF_NORMAL
 from _pygit2 import GIT_FILEMODE_LINK
 from _pygit2 import GIT_BRANCH_LOCAL, GIT_BRANCH_REMOTE, GIT_BRANCH_ALL
+from _pygit2 import GIT_REF_SYMBOLIC
 from _pygit2 import Reference, Tree, Commit, Blob
+from _pygit2 import InvalidSpecError
 
 from .config import Config
 from .errors import check_error
@@ -54,13 +45,13 @@ from .ffi import ffi, C
 from .index import Index
 from .remote import RemoteCollection
 from .blame import Blame
-from .utils import to_bytes, is_string, StrArray
+from .utils import to_bytes, StrArray
 from .submodule import Submodule
 
 
 class BaseRepository(_Repository):
     def __init__(self, backend, *args, **kwargs):
-        super(BaseRepository, self).__init__(backend, *args, **kwargs)
+        super().__init__(backend, *args, **kwargs)
         self._common_init()
 
     def _common_init(self):
@@ -74,6 +65,25 @@ class BaseRepository(_Repository):
         ffi.buffer(repo_cptr)[:] = self._pointer[:]
         self._repo = repo_cptr[0]
 
+    # Backwards compatible ODB access
+    def read(self, *args, **kwargs):
+        """read(oid) -> type, data, size
+
+        Read raw object data from the repository.
+        """
+        return self.odb.read(*args, **kwargs)
+
+    def write(self, *args, **kwargs):
+        """write(type, data) -> Oid
+
+        Write raw object data into the repository. First arg is the object
+        type, the second one a buffer with data. Return the Oid of the created
+        object."""
+        return self.odb.write(*args, **kwargs)
+
+    def __iter__(self):
+        return iter(self.odb)
+
     def lookup_submodule(self, path):
         csub = ffi.new('git_submodule **')
         cpath = ffi.new('char[]', to_bytes(path))
@@ -200,6 +210,30 @@ class BaseRepository(_Repository):
 
         return self.create_reference_symbolic(name, target, force)
 
+    def resolve_refish(self, refish):
+        """Convert a reference-like short name "ref-ish" to a valid
+        (commit, reference) pair.
+
+        If ref-ish points to a commit, the reference element of the result
+        will be None.
+
+        Examples::
+
+            repo.resolve_refish('mybranch')
+            repo.resolve_refish('sometag')
+            repo.resolve_refish('origin/master')
+            repo.resolve_refish('bbb78a9')
+        """
+        try:
+            reference = self.lookup_reference_dwim(refish)
+        except (KeyError, InvalidSpecError):
+            reference = None
+            commit = self.revparse_single(refish)
+        else:
+            commit = reference.peel(Commit)
+
+        return (commit, reference)
+
     #
     # Checkout
     #
@@ -340,6 +374,30 @@ class BaseRepository(_Repository):
     #
     # Diff
     #
+    def __whatever_to_tree_or_blob(self, obj):
+        if obj is None:
+            return None
+
+        # If it's a string, then it has to be valid revspec
+        if isinstance(obj, str):
+            obj = self.revparse_single(obj)
+        elif isinstance(obj, Oid):
+            obj = self[obj]
+
+        # First we try to get to a blob
+        try:
+            obj = obj.peel(Blob)
+        except Exception:
+            # And if that failed, try to get a tree, raising a type
+            # error if that still doesn't work
+            try:
+                obj = obj.peel(Tree)
+            except Exception:
+                raise TypeError('unexpected "%s"' % type(obj))
+
+        return obj
+
+
     def diff(self, a=None, b=None, cached=False, flags=GIT_DIFF_NORMAL,
              context_lines=3, interhunk_lines=0):
         """
@@ -399,29 +457,8 @@ class BaseRepository(_Repository):
         API (Tree.diff_to_tree()) directly.
         """
 
-        def whatever_to_tree_or_blob(obj):
-            if obj is None:
-                return None
-
-            # If it's a string, then it has to be valid revspec
-            if is_string(obj):
-                obj = self.revparse_single(obj)
-
-            # First we try to get to a blob
-            try:
-                obj = obj.peel(Blob)
-            except Exception:
-                # And if that failed, try to get a tree, raising a type
-                # error if that still doesn't work
-                try:
-                    obj = obj.peel(Tree)
-                except Exception:
-                    raise TypeError('unexpected "%s"' % type(obj))
-
-            return obj
-
-        a = whatever_to_tree_or_blob(a)
-        b = whatever_to_tree_or_blob(b)
+        a = self.__whatever_to_tree_or_blob(a)
+        b = self.__whatever_to_tree_or_blob(b)
 
         opt_keys = ['flags', 'context_lines', 'interhunk_lines']
         opt_values = [flags, context_lines, interhunk_lines]
@@ -624,9 +661,9 @@ class BaseRepository(_Repository):
         theirs_ptr = ffi.new('git_commit **')
         cindex = ffi.new('git_index **')
 
-        if is_string(ours) or isinstance(ours, Oid):
+        if isinstance(ours, (str, Oid)):
             ours = self[ours]
-        if is_string(theirs) or isinstance(theirs, Oid):
+        if isinstance(theirs, (str, Oid)):
             theirs = self[theirs]
 
         ours = ours.peel(Commit)
@@ -675,11 +712,11 @@ class BaseRepository(_Repository):
         theirs_ptr = ffi.new('git_tree **')
         cindex = ffi.new('git_index **')
 
-        if is_string(ancestor) or isinstance(ancestor, Oid):
+        if isinstance(ancestor, (str, Oid)):
             ancestor = self[ancestor]
-        if is_string(ours) or isinstance(ours, Oid):
+        if isinstance(ours, (str, Oid)):
             ours = self[ours]
-        if is_string(theirs) or isinstance(theirs, Oid):
+        if isinstance(theirs, (str, Oid)):
             theirs = self[theirs]
 
         ancestor = ancestor.peel(Tree)
@@ -779,7 +816,7 @@ class BaseRepository(_Repository):
 
         result = ffi.new('git_describe_result **')
         if committish:
-            if is_string(committish):
+            if isinstance(committish, str):
                 committish = self.revparse_single(committish)
 
             commit = committish.peel(Commit)
@@ -966,7 +1003,7 @@ class BaseRepository(_Repository):
         if isinstance(treeish, Tree):
             tree = treeish
 
-        if isinstance(treeish, Oid) or is_string(treeish):
+        if isinstance(treeish, (str, Oid)):
             treeish = self[treeish]
 
         # if we don't have a timestamp, try to get it from a commit
@@ -1000,7 +1037,7 @@ class BaseRepository(_Repository):
                 archive.addfile(info)
             else:
                 info.mode = tree[entry.path].filemode
-                archive.addfile(info, StringIO(content))
+                archive.addfile(info, BytesIO(content))
 
     #
     # Ahead-behind, which mostly lives on its own namespace
@@ -1188,8 +1225,14 @@ class Branches(object):
         self[name].delete()
 
     def _valid(self, branch):
-        return (self._commit is None or branch.target == self._commit or
-                self._repository.descendant_of(branch.target, self._commit))
+        if branch.type == GIT_REF_SYMBOLIC:
+            branch = branch.resolve()
+
+        return (
+            self._commit is None
+            or branch.target == self._commit
+            or self._repository.descendant_of(branch.target, self._commit)
+        )
 
     def with_commit(self, commit):
         assert self._commit is None
@@ -1229,14 +1272,17 @@ class References(object):
     def objects(self):
         return self._repository.listall_reference_objects()
 
+    def compress(self):
+        return self._repository.compress_references()
+
 
 class Repository(BaseRepository):
     def __init__(self, path, *args, **kwargs):
-        if not isinstance(path, six.string_types):
+        if not isinstance(path, str):
             path = path.decode('utf-8')
 
         path_backend = init_file_backend(path)
-        super(Repository, self).__init__(backend=path_backend, *args, **kwargs)
+        super().__init__(backend=path_backend, *args, **kwargs)
 
     @classmethod
     def _from_c(cls, ptr, owned):
